# Copyright (C) 2024  The gen-vm-image Project by the Science HPC Center at UCPH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import requests
import time


def download_file(url, output_path, chunk_size=8192):
    response = {}
    try:
        with open(output_path, "wb") as _file:
            response["download_destination"] = output_path
            with requests.get(url, stream=True) as r:
                response["download_src"] = url
                r.raise_for_status()
                total = int(r.headers.get("content-length", 0))
                downloaded = 0
                start_time = time.time()
                for chunk in r.iter_content(chunk_size=chunk_size):
                    downloaded += len(chunk)
                    percentage_progress = (downloaded / total) * 100
                    response["download_progress"] = "{:.2f}%".format(
                        percentage_progress
                    )
                    _file.write(chunk)
                stop_time = time.time()
                response["download_time"] = "{:.2f} seconds".format(
                    stop_time - start_time
                )
    except Exception as e:
        response["msg"] = str(e)
        return False, response
    return True, response



# Copyright (C) 2024  The gen-vm-image Project by the Science HPC Center at UCPH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from gen_vm_image.common.defaults import GENERATED_IMAGE_DIR
from gen_vm_image.common.codes import SUCCESS
from gen_vm_image.cli.build_image import (
    add_build_image_cli_arguments,
    build_architecture,
)


def build_image_cli(commands):
    parser = commands.add_parser(
        "build-image",
        help="Build the images defined in an architecture file.",
    )
    add_build_image_cli_arguments(parser)
    parser.set_defaults(func=corc_build_image_cli_exec)


def corc_build_image_cli_exec(args):
    architecture_path = args.get("architecture_path")
    images_output_directory = args.get("images_output_directory", GENERATED_IMAGE_DIR)
    overwrite = args.get("overwrite", False)
    verbose = args.get("verbose", False)

    return_code, result_dict = build_architecture(
        architecture_path,
        images_output_directory=images_output_directory,
        overwrite=overwrite,
        verbose=verbose,
    )

    if return_code == SUCCESS:
        return True, result_dict
    return False, result_dict



# Copyright (C) 2024  The gen-vm-image Project by the Science HPC Center at UCPH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import yaml
from gen_vm_image.common.codes import (
    PATH_NOT_FOUND_ERROR,
    PATH_NOT_FOUND_ERROR_MSG,
    PATH_LOAD_ERROR,
    PATH_LOAD_ERROR_MSG,
    MISSING_ATTRIBUTE_ERROR,
    MISSING_ATTRIBUTE_ERROR_MSG,
)
from gen_vm_image.utils.io import exists, load


def load_architecture(architecture_path):
    response = {}
    if not exists(architecture_path):
        response["error_code"] = PATH_NOT_FOUND_ERROR
        response["msg"] = PATH_NOT_FOUND_ERROR_MSG.format(
            architecture_path, "Failed to find the architecture file."
        )
        return False, response

    architecture = load(architecture_path, handler=yaml, Loader=yaml.FullLoader)
    if not architecture:
        response["error_code"] = PATH_LOAD_ERROR
        response["msg"] = PATH_LOAD_ERROR_MSG.format(
            architecture_path, "Failed to load the architecture file."
        )
        return False, response
    response["architecture"] = architecture
    return True, response


def correct_architecture_structure(architecture):
    response = {}
    owner = architecture.get("owner", None)
    if not owner:
        response["error_code"] = MISSING_ATTRIBUTE_ERROR
        response["msg"] = MISSING_ATTRIBUTE_ERROR_MSG.format("owner", architecture)
        return False, response

    images = architecture.get("images", None)
    if not images:
        response["error_code"] = MISSING_ATTRIBUTE_ERROR
        response["msg"] = MISSING_ATTRIBUTE_ERROR_MSG.format("images", architecture)
        return False, response

    required_attributes = ["name", "version", "size"]
    for image_name, image_data in images.items():
        for attribute in required_attributes:
            if attribute not in image_data:
                return MISSING_ATTRIBUTE_ERROR, MISSING_ATTRIBUTE_ERROR_MSG.format(
                    attribute, image_name
                )
    return True, response



# Copyright (C) 2024  The gen-vm-image Project by the Science HPC Center at UCPH
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import datetime
import sys


def error_print(*args, **kwargs):
    print(*args, file=sys.stderr, **kwargs)


def to_str(o):
    if hasattr(o, "asdict"):
        return o.asdict()
    if isinstance(o, datetime.datetime):
        return o.__str__()
    if isinstance(o, bytes):
        return o.decode("utf-8")
    return o
